@@ -222,7 +222,7 @@ static void cxg_wait_for_previous_frame(struct cx_gears *cxg)
     {
         hr = ID3D12Fence_SetEventOnCompletion(fence->fence, v, fence->event);
         assert(SUCCEEDED(hr));
-        demo_wait_event(fence->event, INFINITE);
+        demo_wait_event(fence->event);
     }
 
     cxg->rt_idx = demo_swapchain_get_current_back_buffer_index(cxg->swapchain);
@@ -456,13 +456,8 @@ static void cxg_mesh_create(ID3D12Device *device, float inner_radius, float oute
     float r0, r1, r2;
     float angle, da;
 
-    if (!(vertices = calloc(tooth_count, 12 * sizeof(*vertices))))
-        return;
-    if (!(faces = calloc(tooth_count, 20 * sizeof(*faces))))
-    {
-        free(vertices);
-        return;
-    }
+    vertices = calloc(tooth_count, 12 * sizeof(*vertices));
+    faces = calloc(tooth_count, 20 * sizeof(*faces));
 
     r0 = inner_radius;
     r1 = outer_radius - tooth_depth / 2.0f;
@@ -818,7 +813,7 @@ static void cxg_key_press(struct demo_window *window, demo_key key, void *user_d
             cxg->animate = !cxg->animate;
             break;
         case DEMO_KEY_ESCAPE:
-            demo_window_destroy(window);
+            demo_window_destroy_defer(window);
             break;
         case DEMO_KEY_LEFT:
             cxg->theta += M_PI / 36.0f;
