@@ -46,6 +46,8 @@ enum vkd3d_shader_visibility
     VKD3D_SHADER_VISIBILITY_DOMAIN = 3,
     VKD3D_SHADER_VISIBILITY_GEOMETRY = 4,
     VKD3D_SHADER_VISIBILITY_PIXEL = 5,
+    VKD3D_SHADER_VISIBILITY_AMPLIFICATION = 6,
+    VKD3D_SHADER_VISIBILITY_MESH = 7,
 
     VKD3D_SHADER_VISIBILITY_COMPUTE = 1000000000,
 
@@ -57,8 +59,23 @@ typedef uint64_t vkd3d_shader_hash_t;
 enum vkd3d_shader_meta_flags
 {
     VKD3D_SHADER_META_FLAG_REPLACED = 1 << 0,
-    VKD3D_SHADER_META_FLAG_USES_SUBGROUP_SIZE = 1 << 1,
+    VKD3D_SHADER_META_FLAG_USES_SUBGROUP_OPERATIONS = 1 << 1,
     VKD3D_SHADER_META_FLAG_USES_NATIVE_16BIT_OPERATIONS = 1 << 2,
+    VKD3D_SHADER_META_FLAG_USES_FP64 = 1 << 3,
+    VKD3D_SHADER_META_FLAG_USES_INT64 = 1 << 4,
+    VKD3D_SHADER_META_FLAG_USES_STENCIL_EXPORT = 1 << 5,
+    VKD3D_SHADER_META_FLAG_USES_FRAGMENT_FULLY_COVERED = 1 << 6,
+    VKD3D_SHADER_META_FLAG_USES_SHADER_VIEWPORT_INDEX_LAYER = 1 << 7,
+    VKD3D_SHADER_META_FLAG_USES_SPARSE_RESIDENCY = 1 << 8,
+    VKD3D_SHADER_META_FLAG_USES_INT64_ATOMICS = 1 << 9,
+    VKD3D_SHADER_META_FLAG_USES_INT64_ATOMICS_IMAGE = 1 << 10,
+    VKD3D_SHADER_META_FLAG_USES_FRAGMENT_BARYCENTRIC = 1 << 11,
+    VKD3D_SHADER_META_FLAG_USES_SAMPLE_RATE_SHADING = 1 << 12,
+    VKD3D_SHADER_META_FLAG_USES_RASTERIZER_ORDERED_VIEWS = 1 << 13,
+    VKD3D_SHADER_META_FLAG_EMITS_LINES = 1 << 14,
+    VKD3D_SHADER_META_FLAG_EMITS_TRIANGLES = 1 << 15,
+    VKD3D_SHADER_META_FLAG_FORCE_COMPUTE_BARRIER_AFTER_DISPATCH = 1 << 16,
+    VKD3D_SHADER_META_FLAG_EXPORTS_SAMPLE_MASK = 1 << 17,
 };
 
 struct vkd3d_shader_meta
@@ -78,6 +95,14 @@ struct vkd3d_shader_code
     struct vkd3d_shader_meta meta;
 };
 
+struct vkd3d_shader_code_debug
+{
+    const char *debug_entry_point_name;
+};
+
+/* Scans OpCapabilities. */
+void vkd3d_shader_extract_feature_meta(struct vkd3d_shader_code *code);
+
 vkd3d_shader_hash_t vkd3d_shader_hash(const struct vkd3d_shader_code *shader);
 
 enum vkd3d_shader_descriptor_type
@@ -193,9 +218,33 @@ enum vkd3d_shader_interface_flag
     VKD3D_SHADER_INTERFACE_BINDLESS_CBV_AS_STORAGE_BUFFER   = 0x00000002u,
     VKD3D_SHADER_INTERFACE_SSBO_OFFSET_BUFFER               = 0x00000004u,
     VKD3D_SHADER_INTERFACE_TYPED_OFFSET_BUFFER              = 0x00000008u,
-    VKD3D_SHADER_INTERFACE_DESCRIPTOR_QA_BUFFER             = 0x00000010u
+    VKD3D_SHADER_INTERFACE_DESCRIPTOR_QA_BUFFER             = 0x00000010u,
+    /* In this model, use descriptor_size_cbv_srv_uav as array stride for raw VA buffer. */
+    VKD3D_SHADER_INTERFACE_RAW_VA_ALIAS_DESCRIPTOR_BUFFER   = 0x00000020u,
 };
 
+struct vkd3d_shader_stage_io_entry
+{
+    const char *semantic_name;
+    unsigned int semantic_index;
+    unsigned int vk_location;
+    unsigned int vk_component;
+    unsigned int vk_flags;
+};
+
+struct vkd3d_shader_stage_io_map
+{
+    struct vkd3d_shader_stage_io_entry *entries;
+    size_t entries_size;
+    size_t entry_count;
+};
+
+struct vkd3d_shader_stage_io_entry *vkd3d_shader_stage_io_map_append(struct vkd3d_shader_stage_io_map *map,
+        const char *semantic_name, unsigned int semantic_index);
+const struct vkd3d_shader_stage_io_entry *vkd3d_shader_stage_io_map_find(const struct vkd3d_shader_stage_io_map *map,
+        const char *semantic_name, unsigned int semantic_index);
+void vkd3d_shader_stage_io_map_free(struct vkd3d_shader_stage_io_map *map);
+
 struct vkd3d_shader_interface_info
 {
     unsigned int flags; /* vkd3d_shader_interface_flags */
@@ -220,9 +269,16 @@ struct vkd3d_shader_interface_info
     const struct vkd3d_shader_descriptor_binding *descriptor_qa_heap_binding;
 #endif
 
+    const struct vkd3d_shader_stage_io_map *stage_input_map;
+    struct vkd3d_shader_stage_io_map *stage_output_map;
+
     VkShaderStageFlagBits stage;
 
     const struct vkd3d_shader_transform_feedback_info *xfb_info;
+
+    /* Used for either VKD3D_SHADER_INTERFACE_RAW_VA_ALIAS_DESCRIPTOR_BUFFER or local root signatures. */
+    uint32_t descriptor_size_cbv_srv_uav;
+    uint32_t descriptor_size_sampler;
 };
 
 struct vkd3d_shader_descriptor_table
@@ -241,6 +297,7 @@ struct vkd3d_shader_root_constant
 struct vkd3d_shader_root_descriptor
 {
     struct vkd3d_shader_resource_binding *binding;
+    uint32_t raw_va_root_descriptor_index;
 };
 
 struct vkd3d_shader_root_parameter
@@ -262,7 +319,6 @@ struct vkd3d_shader_interface_local_info
     unsigned int shader_record_buffer_count;
     const struct vkd3d_shader_resource_binding *bindings;
     unsigned int binding_count;
-    uint32_t descriptor_size;
 };
 
 struct vkd3d_shader_transform_feedback_element
@@ -308,6 +364,15 @@ enum vkd3d_shader_target_extension
      * all in range, or all out of range. We can implement structured buffer vectorization of vec3,
      * but not byte address buffer. */
     VKD3D_SHADER_TARGET_EXTENSION_ASSUME_PER_COMPONENT_SSBO_ROBUSTNESS,
+    VKD3D_SHADER_TARGET_EXTENSION_BARYCENTRIC_KHR,
+    VKD3D_SHADER_TARGET_EXTENSION_MIN_PRECISION_IS_NATIVE_16BIT,
+    VKD3D_SHADER_TARGET_EXTENSION_SUPPORT_FP16_DENORM_PRESERVE,
+    VKD3D_SHADER_TARGET_EXTENSION_SUPPORT_FP64_DENORM_PRESERVE,
+    VKD3D_SHADER_TARGET_EXTENSION_SUPPORT_SUBGROUP_PARTITIONED_NV,
+    VKD3D_SHADER_TARGET_EXTENSION_COMPUTE_SHADER_DERIVATIVES_NV,
+    VKD3D_SHADER_TARGET_EXTENSION_QUAD_CONTROL_RECONVERGENCE,
+    VKD3D_SHADER_TARGET_EXTENSION_RAW_ACCESS_CHAINS_NV,
+    VKD3D_SHADER_TARGET_EXTENSION_COUNT,
 };
 
 enum vkd3d_shader_quirk
@@ -329,6 +394,42 @@ enum vkd3d_shader_quirk
     /* For Position builtins in Output storage class, emit Invariant decoration.
      * Normally, games have to emit Precise math for position, but if they forget ... */
     VKD3D_SHADER_QUIRK_INVARIANT_POSITION = (1 << 2),
+
+    /* Forces NoContract on every expression that can take it. */
+    VKD3D_SHADER_QUIRK_FORCE_NOCONTRACT_MATH = (1 << 3),
+
+    /* Clamp tessellation factors to a "reasonable" value.
+     * Different flags for different values is a little jank,
+     * but also avoids having to pass down side-channel float values.
+     * Also makes it easier to do app profiles with existing quirk structures. */
+    VKD3D_SHADER_QUIRK_LIMIT_TESS_FACTORS_32 = (1 << 4),
+    VKD3D_SHADER_QUIRK_LIMIT_TESS_FACTORS_16 = (1 << 5),
+    VKD3D_SHADER_QUIRK_LIMIT_TESS_FACTORS_12 = (1 << 6),
+    VKD3D_SHADER_QUIRK_LIMIT_TESS_FACTORS_8 = (1 << 7),
+    VKD3D_SHADER_QUIRK_LIMIT_TESS_FACTORS_4 = (1 << 8),
+
+    /* Force lane count query to return 1.
+     * Can be used to disable buggy subgroup logic that checks for subgroup sizes. */
+    VKD3D_SHADER_QUIRK_FORCE_SUBGROUP_SIZE_1 = (1 << 9),
+
+    /* Enforce a subgroup size of 32 or less. Can be used to work around
+     * issues in shaders that are buggy with large subgroups. */
+    VKD3D_SHADER_QUIRK_FORCE_MAX_WAVE32 = (1 << 10),
+
+    /* For shaders which are bugged when you opt-in to 16-bit. */
+    VKD3D_SHADER_QUIRK_FORCE_MIN16_AS_32BIT = (1 << 11),
+
+    /* Driver workaround hackery. Try to rewrite weird Grads to plain Bias. */
+    VKD3D_SHADER_QUIRK_REWRITE_GRAD_TO_BIAS = (1 << 12),
+
+    /* Driver workarounds. Force loops to not be unrolled with SPIR-V control masks. */
+    VKD3D_SHADER_QUIRK_FORCE_LOOP = (1 << 13),
+
+    /* Requests META_FLAG_FORCE_COMPUTE_BARRIER_AFTER_DISPATCH to be set in shader meta. */
+    VKD3D_SHADER_QUIRK_FORCE_COMPUTE_BARRIER = (1 << 14),
+
+    /* Range check every descriptor heap access with dynamic index and robustness check it. */
+    VKD3D_SHADER_QUIRK_DESCRIPTOR_HEAP_ROBUSTNESS = (1 << 15),
 };
 
 struct vkd3d_shader_quirk_hash
@@ -362,6 +463,10 @@ struct vkd3d_shader_compile_arguments
     const unsigned int *output_swizzles;
     unsigned int output_swizzle_count;
 
+    uint32_t min_subgroup_size;
+    uint32_t max_subgroup_size;
+    bool promote_wave_size_heuristics;
+
     const struct vkd3d_shader_quirk_info *quirks;
 };
 
@@ -454,10 +559,19 @@ enum vkd3d_static_border_color
     VKD3D_STATIC_BORDER_COLOR_TRANSPARENT_BLACK = 0,
     VKD3D_STATIC_BORDER_COLOR_OPAQUE_BLACK = 1,
     VKD3D_STATIC_BORDER_COLOR_OPAQUE_WHITE = 2,
+    VKD3D_STATIC_BORDER_COLOR_OPAQUE_BLACK_UINT = 3,
+    VKD3D_STATIC_BORDER_COLOR_OPAQUE_WHITE_UINT = 4,
 
     VKD3D_FORCE_32_BIT_ENUM(VKD3D_STATIC_BORDER_COLOR),
 };
 
+enum vkd3d_sampler_flags
+{
+    VKD3D_SAMPLER_FLAG_NONE = 0,
+    VKD3D_SAMPLER_FLAG_UINT_BORDER_COLOR = 0x1,
+    VKD3D_SAMPLER_FLAG_NON_NORMALIZED_COORDINATES = 0x2,
+};
+
 struct vkd3d_static_sampler_desc
 {
     enum vkd3d_filter filter;
@@ -475,6 +589,24 @@ struct vkd3d_static_sampler_desc
     enum vkd3d_shader_visibility shader_visibility;
 };
 
+struct vkd3d_static_sampler_desc1
+{
+    enum vkd3d_filter filter;
+    enum vkd3d_texture_address_mode address_u;
+    enum vkd3d_texture_address_mode address_v;
+    enum vkd3d_texture_address_mode address_w;
+    float mip_lod_bias;
+    unsigned int max_anisotropy;
+    enum vkd3d_comparison_func comparison_func;
+    enum vkd3d_static_border_color border_color;
+    float min_lod;
+    float max_lod;
+    unsigned int shader_register;
+    unsigned int register_space;
+    enum vkd3d_shader_visibility shader_visibility;
+    unsigned int flags; /* vkd3d_sampler_flags */
+};
+
 enum vkd3d_descriptor_range_type
 {
     VKD3D_DESCRIPTOR_RANGE_TYPE_SRV = 0,
@@ -622,14 +754,38 @@ struct vkd3d_root_signature_desc1
     enum vkd3d_root_signature_flags flags;
 };
 
+struct vkd3d_root_signature_desc2
+{
+    unsigned int parameter_count;
+    const struct vkd3d_root_parameter1 *parameters;
+    unsigned int static_sampler_count;
+    const struct vkd3d_static_sampler_desc1 *static_samplers;
+    enum vkd3d_root_signature_flags flags;
+};
+
 enum vkd3d_root_signature_version
 {
     VKD3D_ROOT_SIGNATURE_VERSION_1_0 = 0x1,
     VKD3D_ROOT_SIGNATURE_VERSION_1_1 = 0x2,
+    VKD3D_ROOT_SIGNATURE_VERSION_1_2 = 0x3,
 
     VKD3D_FORCE_32_BIT_ENUM(VKD3D_ROOT_SIGNATURE_VERSION),
 };
 
+#define VKD3D_ROOT_SIGNATURE_VERSION_COUNT 3
+static inline unsigned vkd3d_root_signature_version_to_other_index(
+        enum vkd3d_root_signature_version version)
+{
+    assert(version >= VKD3D_ROOT_SIGNATURE_VERSION_1_0 &&
+            version <= VKD3D_ROOT_SIGNATURE_VERSION_1_2);
+    return version - VKD3D_ROOT_SIGNATURE_VERSION_1_0;
+}
+
+static inline bool vkd3d_root_signature_version_is_supported(enum vkd3d_root_signature_version version)
+{
+    return version >= VKD3D_ROOT_SIGNATURE_VERSION_1_0 && version <= VKD3D_ROOT_SIGNATURE_VERSION_1_2;
+}
+
 struct vkd3d_versioned_root_signature_desc
 {
     enum vkd3d_root_signature_version version;
@@ -637,6 +793,7 @@ struct vkd3d_versioned_root_signature_desc
     {
         struct vkd3d_root_signature_desc v_1_0;
         struct vkd3d_root_signature_desc1 v_1_1;
+        struct vkd3d_root_signature_desc2 v_1_2;
     };
 };
 
@@ -645,6 +802,7 @@ enum vkd3d_shader_uav_flag
     VKD3D_SHADER_UAV_FLAG_READ_ACCESS     = 0x00000001,
     VKD3D_SHADER_UAV_FLAG_ATOMIC_COUNTER  = 0x00000002,
     VKD3D_SHADER_UAV_FLAG_ATOMIC_ACCESS   = 0x00000004,
+    VKD3D_SHADER_UAV_FLAG_WRITE_ACCESS    = 0x00000008,
 };
 
 struct vkd3d_shader_scan_info
@@ -656,6 +814,10 @@ struct vkd3d_shader_scan_info
     bool has_side_effects;
     bool needs_late_zs;
     bool discards;
+    bool has_uav_counter;
+    bool declares_globally_coherent_uav;
+    bool requires_thread_group_uav_coherency;
+    bool requires_rov;
     unsigned int patch_vertex_count;
 };
 
@@ -691,6 +853,8 @@ enum vkd3d_sysval_semantic
     VKD3D_SV_TESS_FACTOR_TRIINT        = 14,
     VKD3D_SV_TESS_FACTOR_LINEDET       = 15,
     VKD3D_SV_TESS_FACTOR_LINEDEN       = 16,
+    VKD3D_SV_BARYCENTRICS              = 23,
+    VKD3D_SV_SHADING_RATE              = 24,
 
     VKD3D_FORCE_32_BIT_ENUM(VKD3D_SYSVAL_SEMANTIC),
 };
@@ -743,13 +907,19 @@ struct vkd3d_shader_signature
 #ifndef VKD3D_SHADER_NO_PROTOTYPES
 
 int vkd3d_shader_compile_dxbc(const struct vkd3d_shader_code *dxbc,
-        struct vkd3d_shader_code *spirv, unsigned int compiler_options,
+        struct vkd3d_shader_code *spirv, struct vkd3d_shader_code_debug *spirv_debug,
+        unsigned int compiler_options,
         const struct vkd3d_shader_interface_info *shader_interface_info,
         const struct vkd3d_shader_compile_arguments *compile_args);
 void vkd3d_shader_free_shader_code(struct vkd3d_shader_code *code);
+void vkd3d_shader_free_shader_code_debug(struct vkd3d_shader_code_debug *code);
 
 int vkd3d_shader_parse_root_signature(const struct vkd3d_shader_code *dxbc,
-        struct vkd3d_versioned_root_signature_desc *root_signature);
+        struct vkd3d_versioned_root_signature_desc *root_signature,
+        vkd3d_shader_hash_t *compatibility_hash);
+int vkd3d_shader_parse_root_signature_raw(const char *data, unsigned int data_size,
+        struct vkd3d_versioned_root_signature_desc *desc,
+        vkd3d_shader_hash_t *compatibility_hash);
 void vkd3d_shader_free_root_signature(struct vkd3d_versioned_root_signature_desc *root_signature);
 
 /* FIXME: Add support for returning error messages (ID3DBlob). */
@@ -766,6 +936,8 @@ int vkd3d_shader_parse_input_signature(const struct vkd3d_shader_code *dxbc,
         struct vkd3d_shader_signature *signature);
 int vkd3d_shader_parse_output_signature(const struct vkd3d_shader_code *dxbc,
         struct vkd3d_shader_signature *signature);
+int vkd3d_shader_parse_patch_constant_signature(const struct vkd3d_shader_code *dxbc,
+        struct vkd3d_shader_signature *signature);
 struct vkd3d_shader_signature_element *vkd3d_shader_find_signature_element(
         const struct vkd3d_shader_signature *signature, const char *semantic_name,
         unsigned int semantic_index, unsigned int stream_index);
@@ -775,23 +947,88 @@ void vkd3d_shader_free_shader_signature(struct vkd3d_shader_signature *signature
 struct vkd3d_shader_library_entry_point
 {
     unsigned int identifier;
+    VkShaderStageFlagBits stage;
+
+    uint32_t pipeline_variant_index;
+    uint32_t stage_index;
+
+    /* For implementing the API, since it uses WCHAR despite C++ identifiers being ASCII ... */
     WCHAR *mangled_entry_point;
     WCHAR *plain_entry_point;
+
+    /* Used only for interfacing with dxil-spirv. */
     char *real_entry_point;
-    VkShaderStageFlagBits stage;
+    char *debug_entry_point;
 };
 
-int vkd3d_shader_dxil_append_library_entry_points(
+enum vkd3d_shader_subobject_kind
+{
+    /* Matches DXIL for simplicity. */
+    VKD3D_SHADER_SUBOBJECT_KIND_STATE_OBJECT_CONFIG = 0,
+    VKD3D_SHADER_SUBOBJECT_KIND_GLOBAL_ROOT_SIGNATURE = 1,
+    VKD3D_SHADER_SUBOBJECT_KIND_LOCAL_ROOT_SIGNATURE = 2,
+    VKD3D_SHADER_SUBOBJECT_KIND_SUBOBJECT_TO_EXPORTS_ASSOCIATION = 8,
+    VKD3D_SHADER_SUBOBJECT_KIND_RAYTRACING_SHADER_CONFIG = 9,
+    VKD3D_SHADER_SUBOBJECT_KIND_RAYTRACING_PIPELINE_CONFIG = 10,
+    VKD3D_SHADER_SUBOBJECT_KIND_HIT_GROUP = 11,
+    VKD3D_SHADER_SUBOBJECT_KIND_RAYTRACING_PIPELINE_CONFIG1 = 12,
+};
+
+struct vkd3d_shader_library_subobject
+{
+    enum vkd3d_shader_subobject_kind kind;
+
+    /* All const pointers here point directly to the DXBC blob,
+     * so they do not need to be freed.
+     * Fortunately for us, the C strings are zero-terminated in the blob itself. */
+
+    /* In the blob, ASCII is used as identifier, where API uses wide strings, sigh ...
+     * We need to dup this name for deferred COLLECTIONS, so use wchar instead. */
+    WCHAR *name;
+
+    /* If true, any pointers below are just borrowed. */
+    bool borrowed_payloads;
+
+    union
+    {
+        D3D12_RAYTRACING_PIPELINE_CONFIG1 pipeline_config;
+        D3D12_RAYTRACING_SHADER_CONFIG shader_config;
+        D3D12_STATE_OBJECT_CONFIG object_config;
+
+        /* Duped strings because API wants wide strings for no good reason. */
+        D3D12_HIT_GROUP_DESC hit_group;
+        D3D12_DXIL_SUBOBJECT_TO_EXPORTS_ASSOCIATION association;
+
+        struct
+        {
+            /* Duped because of deferred COLLECTIONS. */
+            void *data;
+            size_t size;
+        } payload;
+    } data;
+};
+
+int vkd3d_shader_dxil_append_library_entry_points_and_subobjects(
         const D3D12_DXIL_LIBRARY_DESC *library_desc,
         unsigned int identifier,
         struct vkd3d_shader_library_entry_point **entry_points,
-        size_t *entry_point_size, size_t *entry_point_count);
+        size_t *entry_point_size, size_t *entry_point_count,
+        struct vkd3d_shader_library_subobject **subobjects,
+        size_t *subobjects_size, size_t *subobjects_count);
 
 void vkd3d_shader_dxil_free_library_entry_points(struct vkd3d_shader_library_entry_point *entry_points, size_t count);
+void vkd3d_shader_dxil_free_library_subobjects(struct vkd3d_shader_library_subobject *subobjects, size_t count);
 
+int vkd3d_shader_dxil_find_global_root_signature_subobject(const void *dxbc, size_t size,
+        struct vkd3d_shader_code *code);
+
+/* export may be a mangled or demangled name.
+ * If RTPSO requests the demangled name, it will likely be demangled, otherwise we forward the mangled name directly.
+ * demangled_export is always a demangled name, for debug purposes. Can be NULL. */
 int vkd3d_shader_compile_dxil_export(const struct vkd3d_shader_code *dxil,
-        const char *export,
+        const char *export, const char *demangled_export,
         struct vkd3d_shader_code *spirv,
+        struct vkd3d_shader_code_debug *spirv_debug,
         const struct vkd3d_shader_interface_info *shader_interface_info,
         const struct vkd3d_shader_interface_local_info *shader_interface_local_info,
         const struct vkd3d_shader_compile_arguments *compiler_args);
@@ -807,13 +1044,15 @@ uint64_t vkd3d_shader_get_revision(void);
  * Function pointer typedefs for vkd3d-shader functions.
  */
 typedef int (*PFN_vkd3d_shader_compile_dxbc)(const struct vkd3d_shader_code *dxbc,
-        struct vkd3d_shader_code *spirv, unsigned int compiler_options,
+        struct vkd3d_shader_code *spirv, struct vkd3d_shader_code_debug *spirv_debug,
+        unsigned int compiler_options,
         const struct vkd3d_shader_interface_info *shader_interface_info,
         const struct vkd3d_shader_compile_arguments *compile_args);
 typedef void (*PFN_vkd3d_shader_free_shader_code)(struct vkd3d_shader_code *code);
 
 typedef int (*PFN_vkd3d_shader_parse_root_signature)(const struct vkd3d_shader_code *dxbc,
-        struct vkd3d_versioned_root_signature_desc *root_signature);
+        struct vkd3d_versioned_root_signature_desc *root_signature,
+        vkd3d_shader_hash_t *compatibility_hash);
 typedef void (*PFN_vkd3d_shader_free_root_signature)(struct vkd3d_versioned_root_signature_desc *root_signature);
 
 typedef int (*PFN_vkd3d_shader_serialize_root_signature)(
@@ -832,6 +1071,19 @@ typedef struct vkd3d_shader_signature_element * (*PFN_vkd3d_shader_find_signatur
         unsigned int semantic_index, unsigned int stream_index);
 typedef void (*PFN_vkd3d_shader_free_shader_signature)(struct vkd3d_shader_signature *signature);
 
+int vkd3d_shader_parse_root_signature_v_1_0(const struct vkd3d_shader_code *dxbc,
+        struct vkd3d_versioned_root_signature_desc *desc,
+        vkd3d_shader_hash_t *compatibility_hash);
+int vkd3d_shader_parse_root_signature_v_1_2(const struct vkd3d_shader_code *dxbc,
+        struct vkd3d_versioned_root_signature_desc *out_desc,
+        vkd3d_shader_hash_t *compatibility_hash);
+int vkd3d_shader_parse_root_signature_v_1_2_from_raw_payload(const struct vkd3d_shader_code *dxbc,
+        struct vkd3d_versioned_root_signature_desc *out_desc,
+        vkd3d_shader_hash_t *compatibility_hash);
+
+vkd3d_shader_hash_t vkd3d_root_signature_v_1_2_compute_layout_compat_hash(
+        const struct vkd3d_root_signature_desc2 *desc);
+
 #ifdef __cplusplus
 }
 #endif  /* __cplusplus */
