@@ -19,6 +19,9 @@
 #define VKD3D_DBG_CHANNEL VKD3D_DBG_CHANNEL_SHADER
 
 #include "vkd3d_shader_private.h"
+#include "vkd3d_string.h"
+
+#include "vkd3d_platform.h"
 
 #include <stdio.h>
 #include <inttypes.h>
@@ -81,13 +84,13 @@ err:
 bool vkd3d_shader_replace(vkd3d_shader_hash_t hash, const void **data, size_t *size)
 {
     static bool enabled = true;
+    char path[VKD3D_PATH_MAX];
     char filename[1024];
-    const char *path;
 
     if (!enabled)
         return false;
 
-    if (!(path = getenv("VKD3D_SHADER_OVERRIDE")))
+    if (!vkd3d_get_env_var("VKD3D_SHADER_OVERRIDE", path, sizeof(path)))
     {
         enabled = false;
         return false;
@@ -100,13 +103,13 @@ bool vkd3d_shader_replace(vkd3d_shader_hash_t hash, const void **data, size_t *s
 bool vkd3d_shader_replace_export(vkd3d_shader_hash_t hash, const void **data, size_t *size, const char *export)
 {
     static bool enabled = true;
+    char path[VKD3D_PATH_MAX];
     char filename[1024];
-    const char *path;
 
     if (!enabled)
         return false;
 
-    if (!(path = getenv("VKD3D_SHADER_OVERRIDE")))
+    if (!vkd3d_get_env_var("VKD3D_SHADER_OVERRIDE", path, sizeof(path)))
     {
         enabled = false;
         return false;
@@ -119,12 +122,12 @@ bool vkd3d_shader_replace_export(vkd3d_shader_hash_t hash, const void **data, si
 void vkd3d_shader_dump_shader(vkd3d_shader_hash_t hash, const struct vkd3d_shader_code *shader, const char *ext)
 {
     static bool enabled = true;
-    const char *path;
+    char path[VKD3D_PATH_MAX];
 
     if (!enabled)
         return;
 
-    if (!(path = getenv("VKD3D_SHADER_DUMP_PATH")))
+    if (!vkd3d_get_env_var("VKD3D_SHADER_DUMP_PATH", path, sizeof(path)))
     {
         enabled = false;
         return;
@@ -136,12 +139,12 @@ void vkd3d_shader_dump_shader(vkd3d_shader_hash_t hash, const struct vkd3d_shade
 void vkd3d_shader_dump_spirv_shader(vkd3d_shader_hash_t hash, const struct vkd3d_shader_code *shader)
 {
     static bool enabled = true;
-    const char *path;
+    char path[VKD3D_PATH_MAX];
 
     if (!enabled)
         return;
 
-    if (!(path = getenv("VKD3D_SHADER_DUMP_PATH")))
+    if (!vkd3d_get_env_var("VKD3D_SHADER_DUMP_PATH", path, sizeof(path)))
     {
         enabled = false;
         return;
@@ -154,13 +157,13 @@ void vkd3d_shader_dump_spirv_shader_export(vkd3d_shader_hash_t hash, const struc
         const char *export)
 {
     static bool enabled = true;
-    const char *path;
+    char path[VKD3D_PATH_MAX];
     char tag[1024];
 
     if (!enabled)
         return;
 
-    if (!(path = getenv("VKD3D_SHADER_DUMP_PATH")))
+    if (!vkd3d_get_env_var("VKD3D_SHADER_DUMP_PATH", path, sizeof(path)))
     {
         enabled = false;
         return;
@@ -236,6 +239,7 @@ struct vkd3d_shader_scan_entry
     struct hash_map_entry entry;
     struct vkd3d_shader_scan_key key;
     unsigned int flags;
+    unsigned required_components;
 };
 
 static uint32_t vkd3d_shader_scan_entry_hash(const void *key)
@@ -254,32 +258,71 @@ static bool vkd3d_shader_scan_entry_compare(const void *key, const struct hash_m
 unsigned int vkd3d_shader_scan_get_register_flags(const struct vkd3d_shader_scan_info *scan_info,
         enum vkd3d_shader_register_type type, unsigned int id)
 {
+    const struct vkd3d_shader_scan_entry *e;
     struct vkd3d_shader_scan_key key;
-    struct hash_map_entry *e;
 
     key.register_type = type;
     key.register_id = id;
 
-    e = hash_map_find(&scan_info->register_map, &key);
+    e = (const struct vkd3d_shader_scan_entry *)hash_map_find(&scan_info->register_map, &key);
     return e ? e->flags : 0u;
 }
 
+unsigned int vkd3d_shader_scan_get_idxtemp_components(const struct vkd3d_shader_scan_info *scan_info,
+        const struct vkd3d_shader_register *reg)
+{
+    const struct vkd3d_shader_scan_entry *e;
+    struct vkd3d_shader_scan_key key;
+
+    key.register_type = reg->type;
+    key.register_id = reg->idx[0].offset;
+
+    e = (const struct vkd3d_shader_scan_entry *)hash_map_find(&scan_info->register_map, &key);
+    return e ? e->required_components : 4u;
+}
+
 static void vkd3d_shader_scan_set_register_flags(struct vkd3d_shader_scan_info *scan_info,
         enum vkd3d_shader_register_type type, unsigned int id, unsigned int flags)
 {
     struct vkd3d_shader_scan_entry entry;
+    struct vkd3d_shader_scan_entry *e;
     struct vkd3d_shader_scan_key key;
-    struct hash_map_entry *e;
 
     key.register_type = type;
     key.register_id = id;
 
-    if ((e = hash_map_find(&scan_info->register_map, &key)))
+    if ((e = (struct vkd3d_shader_scan_entry *)hash_map_find(&scan_info->register_map, &key)))
+    {
         e->flags |= flags;
+    }
     else
     {
         entry.key = key;
         entry.flags = flags;
+        entry.required_components = 0;
+        hash_map_insert(&scan_info->register_map, &key, &entry.entry);
+    }
+}
+
+static void vkd3d_shader_scan_record_idxtemp_components(struct vkd3d_shader_scan_info *scan_info,
+        const struct vkd3d_shader_register *reg, unsigned int required_components)
+{
+    struct vkd3d_shader_scan_entry entry;
+    struct vkd3d_shader_scan_entry *e;
+    struct vkd3d_shader_scan_key key;
+
+    key.register_type = reg->type;
+    key.register_id = reg->idx[0].offset;
+
+    if ((e = (struct vkd3d_shader_scan_entry *)hash_map_find(&scan_info->register_map, &key)))
+    {
+        e->required_components = max(required_components, e->required_components);
+    }
+    else
+    {
+        entry.key = key;
+        entry.flags = 0;
+        entry.required_components = required_components;
         hash_map_insert(&scan_info->register_map, &key, &entry.entry);
     }
 }
@@ -293,7 +336,7 @@ static void vkd3d_shader_scan_init(struct vkd3d_shader_scan_info *scan_info)
 
 static void vkd3d_shader_scan_destroy(struct vkd3d_shader_scan_info *scan_info)
 {
-    hash_map_clear(&scan_info->register_map);
+    hash_map_free(&scan_info->register_map);
 }
 
 static int vkd3d_shader_validate_shader_type(enum vkd3d_shader_type type, VkShaderStageFlagBits stages)
@@ -320,7 +363,9 @@ static int vkd3d_shader_validate_shader_type(enum vkd3d_shader_type type, VkShad
 }
 
 int vkd3d_shader_compile_dxbc(const struct vkd3d_shader_code *dxbc,
-        struct vkd3d_shader_code *spirv, unsigned int compiler_options,
+        struct vkd3d_shader_code *spirv,
+        struct vkd3d_shader_code_debug *spirv_debug,
+        unsigned int compiler_options,
         const struct vkd3d_shader_interface_info *shader_interface_info,
         const struct vkd3d_shader_compile_arguments *compile_args)
 {
@@ -340,7 +385,7 @@ int vkd3d_shader_compile_dxbc(const struct vkd3d_shader_code *dxbc,
     /* DXIL is handled externally through dxil-spirv. */
     if (shader_is_dxil(dxbc->code, dxbc->size))
     {
-        return vkd3d_shader_compile_dxil(dxbc, spirv, shader_interface_info, compile_args);
+        return vkd3d_shader_compile_dxil(dxbc, spirv, spirv_debug, shader_interface_info, compile_args);
     }
 
     memset(&spirv->meta, 0, sizeof(spirv->meta));
@@ -353,6 +398,8 @@ int vkd3d_shader_compile_dxbc(const struct vkd3d_shader_code *dxbc,
         return VKD3D_OK;
     }
 
+    vkd3d_shader_dump_shader(hash, dxbc, "dxbc");
+
     vkd3d_shader_scan_init(&scan_info);
 
     if ((ret = vkd3d_shader_scan_dxbc(dxbc, &scan_info)) < 0)
@@ -374,15 +421,11 @@ int vkd3d_shader_compile_dxbc(const struct vkd3d_shader_code *dxbc,
         if ((ret = vkd3d_shader_validate_shader_type(parser.shader_version.type, shader_interface_info->stage)) < 0)
         {
             vkd3d_shader_scan_destroy(&scan_info);
+            vkd3d_shader_parser_destroy(&parser);
             return ret;
         }
     }
 
-    vkd3d_shader_dump_shader(hash, dxbc, "dxbc");
-
-    if (TRACE_ON())
-        vkd3d_shader_trace(parser.data);
-
     if (!(spirv_compiler = vkd3d_dxbc_compiler_create(&parser.shader_version,
             &parser.shader_desc, compiler_options, shader_interface_info, compile_args, &scan_info,
             spirv->meta.hash)))
@@ -416,6 +459,9 @@ int vkd3d_shader_compile_dxbc(const struct vkd3d_shader_code *dxbc,
     if (ret == 0)
         vkd3d_shader_dump_spirv_shader(hash, spirv);
 
+    if (spirv_debug)
+        memset(spirv_debug, 0, sizeof(*spirv_debug));
+
     vkd3d_dxbc_compiler_destroy(spirv_compiler);
     vkd3d_shader_scan_destroy(&scan_info);
     vkd3d_shader_parser_destroy(&parser);
@@ -457,6 +503,13 @@ static void vkd3d_shader_scan_record_uav_read(struct vkd3d_shader_scan_info *sca
             reg->idx[0].offset, VKD3D_SHADER_UAV_FLAG_READ_ACCESS);
 }
 
+static void vkd3d_shader_scan_record_uav_write(struct vkd3d_shader_scan_info *scan_info,
+        const struct vkd3d_shader_register *reg)
+{
+    vkd3d_shader_scan_set_register_flags(scan_info, VKD3DSPR_UAV,
+            reg->idx[0].offset, VKD3D_SHADER_UAV_FLAG_WRITE_ACCESS);
+}
+
 static void vkd3d_shader_scan_record_uav_atomic(struct vkd3d_shader_scan_info *scan_info,
         const struct vkd3d_shader_register *reg)
 {
@@ -475,6 +528,7 @@ static void vkd3d_shader_scan_record_uav_counter(struct vkd3d_shader_scan_info *
         const struct vkd3d_shader_register *reg)
 {
     scan_info->has_side_effects = true;
+    scan_info->has_uav_counter = true;
     vkd3d_shader_scan_set_register_flags(scan_info, VKD3DSPR_UAV,
             reg->idx[0].offset, VKD3D_SHADER_UAV_FLAG_ATOMIC_COUNTER);
 }
@@ -530,10 +584,42 @@ static void vkd3d_shader_scan_instruction(struct vkd3d_shader_scan_info *scan_in
         case VKD3DSIH_DCL_INPUT_CONTROL_POINT_COUNT:
             scan_info->patch_vertex_count = instruction->declaration.count;
             break;
+        case VKD3DSIH_DCL_UAV_RAW:
+        case VKD3DSIH_DCL_UAV_STRUCTURED:
+        case VKD3DSIH_DCL_UAV_TYPED:
+            /* See test_memory_model_uav_coherent_thread_group() for details. */
+            if (instruction->flags & VKD3DSUF_GLOBALLY_COHERENT)
+                scan_info->declares_globally_coherent_uav = true;
+            if (instruction->flags & VKD3DSUF_RASTERIZER_ORDERED)
+                scan_info->requires_rov = true;
+            break;
+        case VKD3DSIH_SYNC:
+            /* See test_memory_model_uav_coherent_thread_group() for details. */
+            if (instruction->flags & (VKD3DSSF_UAV_MEMORY_LOCAL | VKD3DSSF_UAV_MEMORY_GLOBAL))
+                scan_info->requires_thread_group_uav_coherency = true;
+            break;
         default:
             break;
     }
 
+    /* If we do nothing, we will have to assume that IDXTEMP is an array of vec4.
+     * This is problematic for performance if shader only accesses the first 1, 2 or 3 components.
+     * The dcl_indexableTemp instruction specifies number of components but FXC does not seem to
+     * care, so we have to analyze write masks instead. */
+    for (i = 0; i < instruction->dst_count; ++i)
+    {
+        if (instruction->dst[i].reg.type == VKD3DSPR_IDXTEMP)
+        {
+            unsigned int write_mask, required_components;
+            write_mask = instruction->dst[i].write_mask;
+            write_mask |= write_mask >> 2;
+            write_mask |= write_mask >> 1;
+            required_components = vkd3d_write_mask_component_count(write_mask);
+            vkd3d_shader_scan_record_idxtemp_components(scan_info,
+                    &instruction->dst[i].reg, required_components);
+        }
+    }
+
     if (vkd3d_shader_instruction_is_uav_read(instruction))
     {
         is_atomic = vkd3d_shader_instruction_is_uav_atomic(instruction);
@@ -559,7 +645,12 @@ static void vkd3d_shader_scan_instruction(struct vkd3d_shader_scan_info *scan_in
     }
 
     if (vkd3d_shader_instruction_is_uav_write(instruction))
+    {
         scan_info->has_side_effects = true;
+        for (i = 0; i < instruction->dst_count; ++i)
+            if (instruction->dst[i].reg.type == VKD3DSPR_UAV)
+                vkd3d_shader_scan_record_uav_write(scan_info, &instruction->dst[i].reg);
+    }
 
     if (vkd3d_shader_instruction_is_uav_counter(instruction))
         vkd3d_shader_scan_record_uav_counter(scan_info, &instruction->src[0].reg);
@@ -611,6 +702,14 @@ void vkd3d_shader_free_shader_code(struct vkd3d_shader_code *shader_code)
     vkd3d_free((void *)shader_code->code);
 }
 
+void vkd3d_shader_free_shader_code_debug(struct vkd3d_shader_code_debug *shader_code)
+{
+    if (!shader_code)
+        return;
+
+    vkd3d_free((void *)shader_code->debug_entry_point_name);
+}
+
 static void vkd3d_shader_free_root_signature_v_1_0(struct vkd3d_root_signature_desc *root_signature)
 {
     unsigned int i;
@@ -645,6 +744,23 @@ static void vkd3d_shader_free_root_signature_v_1_1(struct vkd3d_root_signature_d
     memset(root_signature, 0, sizeof(*root_signature));
 }
 
+static void vkd3d_shader_free_root_signature_v_1_2(struct vkd3d_root_signature_desc2 *root_signature)
+{
+    unsigned int i;
+
+    for (i = 0; i < root_signature->parameter_count; ++i)
+    {
+        const struct vkd3d_root_parameter1 *parameter = &root_signature->parameters[i];
+
+        if (parameter->parameter_type == VKD3D_ROOT_PARAMETER_TYPE_DESCRIPTOR_TABLE)
+            vkd3d_free((void *)parameter->descriptor_table.descriptor_ranges);
+    }
+    vkd3d_free((void *)root_signature->parameters);
+    vkd3d_free((void *)root_signature->static_samplers);
+
+    memset(root_signature, 0, sizeof(*root_signature));
+}
+
 void vkd3d_shader_free_root_signature(struct vkd3d_versioned_root_signature_desc *desc)
 {
     if (desc->version == VKD3D_ROOT_SIGNATURE_VERSION_1_0)
@@ -655,6 +771,10 @@ void vkd3d_shader_free_root_signature(struct vkd3d_versioned_root_signature_desc
     {
         vkd3d_shader_free_root_signature_v_1_1(&desc->v_1_1);
     }
+    else if (desc->version == VKD3D_ROOT_SIGNATURE_VERSION_1_2)
+    {
+        vkd3d_shader_free_root_signature_v_1_2(&desc->v_1_2);
+    }
     else if (desc->version)
     {
         FIXME("Unknown version %#x.\n", desc->version);
@@ -680,6 +800,14 @@ int vkd3d_shader_parse_output_signature(const struct vkd3d_shader_code *dxbc,
     return shader_parse_output_signature(dxbc->code, dxbc->size, signature);
 }
 
+int vkd3d_shader_parse_patch_constant_signature(const struct vkd3d_shader_code *dxbc,
+        struct vkd3d_shader_signature *signature)
+{
+    TRACE("dxbc {%p, %zu}, signature %p.\n", dxbc->code, dxbc->size, signature);
+
+    return shader_parse_patch_constant_signature(dxbc->code, dxbc->size, signature);
+}
+
 struct vkd3d_shader_signature_element *vkd3d_shader_find_signature_element(
         const struct vkd3d_shader_signature *signature, const char *semantic_name,
         unsigned int semantic_index, unsigned int stream_index)
@@ -745,3 +873,158 @@ uint64_t vkd3d_shader_get_revision(void)
      * by vkd3d-proton Git hash. */
     return 1;
 }
+
+struct vkd3d_shader_stage_io_entry *vkd3d_shader_stage_io_map_append(struct vkd3d_shader_stage_io_map *map,
+        const char *semantic_name, unsigned int semantic_index)
+{
+    struct vkd3d_shader_stage_io_entry *e;
+
+    if (vkd3d_shader_stage_io_map_find(map, semantic_name, semantic_index))
+        return NULL;
+
+    if (!vkd3d_array_reserve((void **)&map->entries, &map->entries_size,
+            map->entry_count + 1, sizeof(*map->entries)))
+        return NULL;
+
+    e = &map->entries[map->entry_count++];
+    e->semantic_name = vkd3d_strdup(semantic_name);
+    e->semantic_index = semantic_index;
+    return e;
+}
+
+const struct vkd3d_shader_stage_io_entry *vkd3d_shader_stage_io_map_find(const struct vkd3d_shader_stage_io_map *map,
+        const char *semantic_name, unsigned int semantic_index)
+{
+    unsigned int i;
+
+    for (i = 0; i < map->entry_count; i++)
+    {
+        struct vkd3d_shader_stage_io_entry *e = &map->entries[i];
+
+        if (!strcmp(e->semantic_name, semantic_name) && e->semantic_index == semantic_index)
+            return e;
+    }
+
+    return NULL;
+}
+
+void vkd3d_shader_stage_io_map_free(struct vkd3d_shader_stage_io_map *map)
+{
+    unsigned int i;
+
+    for (i = 0; i < map->entry_count; i++)
+        vkd3d_free((void *)map->entries[i].semantic_name);
+
+    vkd3d_free(map->entries);
+    memset(map, 0, sizeof(*map));
+}
+
+static int vkd3d_shader_parse_root_signature_for_version(const struct vkd3d_shader_code *dxbc,
+        struct vkd3d_versioned_root_signature_desc *out_desc,
+        enum vkd3d_root_signature_version target_version,
+        bool raw_payload,
+        vkd3d_shader_hash_t *compatibility_hash)
+{
+    struct vkd3d_versioned_root_signature_desc desc, converted_desc;
+    int ret;
+
+    if (raw_payload)
+    {
+        if ((ret = vkd3d_shader_parse_root_signature_raw(dxbc->code, dxbc->size, &desc, compatibility_hash)) < 0)
+        {
+            WARN("Failed to parse root signature, vkd3d result %d.\n", ret);
+            return ret;
+        }
+    }
+    else
+    {
+        if ((ret = vkd3d_shader_parse_root_signature(dxbc, &desc, compatibility_hash)) < 0)
+        {
+            WARN("Failed to parse root signature, vkd3d result %d.\n", ret);
+            return ret;
+        }
+    }
+
+    if (desc.version == target_version)
+    {
+        *out_desc = desc;
+    }
+    else
+    {
+        ret = vkd3d_shader_convert_root_signature(&converted_desc, target_version, &desc);
+        vkd3d_shader_free_root_signature(&desc);
+        if (ret < 0)
+        {
+            WARN("Failed to convert from version %#x, vkd3d result %d.\n", desc.version, ret);
+            return ret;
+        }
+
+        *out_desc = converted_desc;
+    }
+
+    return ret;
+}
+
+int vkd3d_shader_parse_root_signature_v_1_0(const struct vkd3d_shader_code *dxbc,
+        struct vkd3d_versioned_root_signature_desc *out_desc,
+        vkd3d_shader_hash_t *compatibility_hash)
+{
+    return vkd3d_shader_parse_root_signature_for_version(dxbc, out_desc, VKD3D_ROOT_SIGNATURE_VERSION_1_0, false,
+            compatibility_hash);
+}
+
+int vkd3d_shader_parse_root_signature_v_1_2(const struct vkd3d_shader_code *dxbc,
+        struct vkd3d_versioned_root_signature_desc *out_desc,
+        vkd3d_shader_hash_t *compatibility_hash)
+{
+    return vkd3d_shader_parse_root_signature_for_version(dxbc, out_desc, VKD3D_ROOT_SIGNATURE_VERSION_1_2, false,
+            compatibility_hash);
+}
+
+int vkd3d_shader_parse_root_signature_v_1_2_from_raw_payload(const struct vkd3d_shader_code *dxbc,
+        struct vkd3d_versioned_root_signature_desc *out_desc,
+        vkd3d_shader_hash_t *compatibility_hash)
+{
+    return vkd3d_shader_parse_root_signature_for_version(dxbc, out_desc, VKD3D_ROOT_SIGNATURE_VERSION_1_2, true,
+            compatibility_hash);
+}
+
+vkd3d_shader_hash_t vkd3d_root_signature_v_1_2_compute_layout_compat_hash(
+        const struct vkd3d_root_signature_desc2 *desc)
+{
+    vkd3d_shader_hash_t hash = hash_fnv1_init();
+    uint32_t i;
+
+    hash = hash_fnv1_iterate_u32(hash, desc->static_sampler_count);
+    hash = hash_fnv1_iterate_u32(hash, desc->parameter_count);
+    hash = hash_fnv1_iterate_u32(hash, desc->flags & D3D12_ROOT_SIGNATURE_FLAG_LOCAL_ROOT_SIGNATURE);
+
+    for (i = 0; i < desc->parameter_count; i++)
+    {
+        hash = hash_fnv1_iterate_u32(hash, desc->parameters[i].parameter_type);
+        if (desc->parameters[i].parameter_type == VKD3D_ROOT_PARAMETER_TYPE_32BIT_CONSTANTS)
+            hash = hash_fnv1_iterate_u32(hash, desc->parameters[i].constants.value_count);
+        else
+            hash = hash_fnv1_iterate_u32(hash, 0);
+    }
+
+    for (i = 0; i < desc->static_sampler_count; i++)
+    {
+        const struct vkd3d_static_sampler_desc1 *sampler = &desc->static_samplers[i];
+        /* Ignore space / register since those don't affect VkPipelineLayout. */
+        hash = hash_fnv1_iterate_u32(hash, sampler->flags);
+        hash = hash_fnv1_iterate_u32(hash, sampler->shader_visibility);
+        hash = hash_fnv1_iterate_u32(hash, sampler->max_anisotropy);
+        hash = hash_fnv1_iterate_u32(hash, sampler->border_color);
+        hash = hash_fnv1_iterate_u32(hash, sampler->comparison_func);
+        hash = hash_fnv1_iterate_u32(hash, sampler->address_u);
+        hash = hash_fnv1_iterate_u32(hash, sampler->address_v);
+        hash = hash_fnv1_iterate_u32(hash, sampler->address_w);
+        hash = hash_fnv1_iterate_u32(hash, sampler->filter);
+        hash = hash_fnv1_iterate_f32(hash, sampler->min_lod);
+        hash = hash_fnv1_iterate_f32(hash, sampler->max_lod);
+        hash = hash_fnv1_iterate_f32(hash, sampler->mip_lod_bias);
+    }
+
+    return hash;
+}
