@@ -75,6 +75,9 @@ static HRESULT STDMETHODCALLTYPE d3d12_bundle_allocator_QueryInterface(ID3D12Com
 {
     TRACE("iface %p, riid %s, object %p.\n", iface, debugstr_guid(riid), object);
 
+    if (!object)
+        return E_POINTER;
+
     if (IsEqualGUID(riid, &IID_ID3D12CommandAllocator)
             || IsEqualGUID(riid, &IID_ID3D12Pageable)
             || IsEqualGUID(riid, &IID_ID3D12DeviceChild)
@@ -255,6 +258,9 @@ static HRESULT STDMETHODCALLTYPE d3d12_bundle_QueryInterface(d3d12_command_list_
 {
     TRACE("iface %p, iid %s, object %p.\n", iface, debugstr_guid(iid), object);
 
+    if (!object)
+        return E_POINTER;
+
     if (IsEqualGUID(iid, &IID_ID3D12GraphicsCommandList)
             || IsEqualGUID(iid, &IID_ID3D12GraphicsCommandList1)
             || IsEqualGUID(iid, &IID_ID3D12GraphicsCommandList2)
@@ -262,12 +268,15 @@ static HRESULT STDMETHODCALLTYPE d3d12_bundle_QueryInterface(d3d12_command_list_
             || IsEqualGUID(iid, &IID_ID3D12GraphicsCommandList4)
             || IsEqualGUID(iid, &IID_ID3D12GraphicsCommandList5)
             || IsEqualGUID(iid, &IID_ID3D12GraphicsCommandList6)
+            || IsEqualGUID(iid, &IID_ID3D12GraphicsCommandList7)
+            || IsEqualGUID(iid, &IID_ID3D12GraphicsCommandList8)
+            || IsEqualGUID(iid, &IID_ID3D12GraphicsCommandList9)
             || IsEqualGUID(iid, &IID_ID3D12CommandList)
             || IsEqualGUID(iid, &IID_ID3D12DeviceChild)
             || IsEqualGUID(iid, &IID_ID3D12Object)
             || IsEqualGUID(iid, &IID_IUnknown))
     {
-        ID3D12GraphicsCommandList6_AddRef(iface);
+        ID3D12GraphicsCommandList9_AddRef(iface);
         *object = iface;
         return S_OK;
     }
@@ -430,7 +439,7 @@ static void d3d12_bundle_exec_draw_instanced(d3d12_command_list_iface *list, con
 {
     const struct d3d12_draw_instanced_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_DrawInstanced(list, args->vertex_count,
+    ID3D12GraphicsCommandList9_DrawInstanced(list, args->vertex_count,
             args->instance_count, args->first_vertex, args->first_instance);
 }
 
@@ -467,7 +476,7 @@ static void d3d12_bundle_exec_draw_indexed_instanced(d3d12_command_list_iface *l
 {
     const struct d3d12_draw_indexed_instanced_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_DrawIndexedInstanced(list, args->index_count,
+    ID3D12GraphicsCommandList9_DrawIndexedInstanced(list, args->index_count,
             args->instance_count, args->first_index, args->vertex_offset,
             args->first_instance);
 }
@@ -502,7 +511,7 @@ static void d3d12_bundle_exec_dispatch(d3d12_command_list_iface *list, const voi
 {
     const struct d3d12_dispatch_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_Dispatch(list, args->x, args->y, args->z);
+    ID3D12GraphicsCommandList9_Dispatch(list, args->x, args->y, args->z);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_Dispatch(d3d12_command_list_iface *iface,
@@ -570,7 +579,7 @@ static void d3d12_bundle_exec_ia_set_primitive_topology(d3d12_command_list_iface
 {
     const struct d3d12_ia_set_primitive_topology_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_IASetPrimitiveTopology(list, args->topology);
+    ID3D12GraphicsCommandList9_IASetPrimitiveTopology(list, args->topology);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_IASetPrimitiveTopology(d3d12_command_list_iface *iface,
@@ -607,7 +616,7 @@ static void d3d12_bundle_exec_om_set_blend_factor(d3d12_command_list_iface *list
 {
     const struct d3d12_om_set_blend_factor_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_OMSetBlendFactor(list, args->blend_factor);
+    ID3D12GraphicsCommandList9_OMSetBlendFactor(list, args->blend_factor);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_OMSetBlendFactor(d3d12_command_list_iface *iface,
@@ -635,7 +644,7 @@ static void d3d12_bundle_exec_om_set_stencil_ref(d3d12_command_list_iface *list,
 {
     const struct d3d12_om_set_stencil_ref_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_OMSetStencilRef(list, args->stencil_ref);
+    ID3D12GraphicsCommandList9_OMSetStencilRef(list, args->stencil_ref);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_OMSetStencilRef(d3d12_command_list_iface *iface,
@@ -660,7 +669,7 @@ static void d3d12_bundle_exec_set_pipeline_state(d3d12_command_list_iface *list,
 {
     const struct d3d12_set_pipeline_state_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetPipelineState(list, args->pipeline_state);
+    ID3D12GraphicsCommandList9_SetPipelineState(list, args->pipeline_state);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_SetPipelineState(d3d12_command_list_iface *iface,
@@ -704,7 +713,7 @@ static void d3d12_bundle_exec_set_compute_root_signature(d3d12_command_list_ifac
 {
     const struct d3d12_set_root_signature_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetComputeRootSignature(list, args->root_signature);
+    ID3D12GraphicsCommandList9_SetComputeRootSignature(list, args->root_signature);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_SetComputeRootSignature(d3d12_command_list_iface *iface,
@@ -723,7 +732,7 @@ static void d3d12_bundle_exec_set_graphics_root_signature(d3d12_command_list_ifa
 {
     const struct d3d12_set_root_signature_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetGraphicsRootSignature(list, args->root_signature);
+    ID3D12GraphicsCommandList9_SetGraphicsRootSignature(list, args->root_signature);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_SetGraphicsRootSignature(d3d12_command_list_iface *iface,
@@ -749,7 +758,7 @@ static void d3d12_bundle_exec_set_compute_root_descriptor_table(d3d12_command_li
 {
     const struct d3d12_set_root_descriptor_table_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetComputeRootDescriptorTable(list, args->parameter_index, args->base_descriptor);
+    ID3D12GraphicsCommandList9_SetComputeRootDescriptorTable(list, args->parameter_index, args->base_descriptor);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_SetComputeRootDescriptorTable(d3d12_command_list_iface *iface,
@@ -770,7 +779,7 @@ static void d3d12_bundle_exec_set_graphics_root_descriptor_table(d3d12_command_l
 {
     const struct d3d12_set_root_descriptor_table_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetGraphicsRootDescriptorTable(list, args->parameter_index, args->base_descriptor);
+    ID3D12GraphicsCommandList9_SetGraphicsRootDescriptorTable(list, args->parameter_index, args->base_descriptor);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_SetGraphicsRootDescriptorTable(d3d12_command_list_iface *iface,
@@ -799,7 +808,7 @@ static void d3d12_bundle_exec_set_compute_root_32bit_constant(d3d12_command_list
 {
     const struct d3d12_set_root_32bit_constant_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetComputeRoot32BitConstant(list, args->parameter_index, args->data, args->offset);
+    ID3D12GraphicsCommandList9_SetComputeRoot32BitConstant(list, args->parameter_index, args->data, args->offset);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_SetComputeRoot32BitConstant(d3d12_command_list_iface *iface,
@@ -821,7 +830,7 @@ static void d3d12_bundle_exec_set_graphics_root_32bit_constant(d3d12_command_lis
 {
     const struct d3d12_set_root_32bit_constant_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetGraphicsRoot32BitConstant(list, args->parameter_index, args->data, args->offset);
+    ID3D12GraphicsCommandList9_SetGraphicsRoot32BitConstant(list, args->parameter_index, args->data, args->offset);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_SetGraphicsRoot32BitConstant(d3d12_command_list_iface *iface,
@@ -852,7 +861,7 @@ static void d3d12_bundle_exec_set_compute_root_32bit_constants(d3d12_command_lis
 {
     const struct d3d12_set_root_32bit_constants_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetComputeRoot32BitConstants(list, args->parameter_index,
+    ID3D12GraphicsCommandList9_SetComputeRoot32BitConstants(list, args->parameter_index,
             args->constant_count, args->data, args->offset);
 }
 
@@ -880,7 +889,7 @@ static void d3d12_bundle_exec_set_graphics_root_32bit_constants(d3d12_command_li
 {
     const struct d3d12_set_root_32bit_constants_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetGraphicsRoot32BitConstants(list, args->parameter_index,
+    ID3D12GraphicsCommandList9_SetGraphicsRoot32BitConstants(list, args->parameter_index,
             args->constant_count, args->data, args->offset);
 }
 
@@ -915,7 +924,7 @@ static void d3d12_bundle_exec_set_compute_root_cbv(d3d12_command_list_iface *lis
 {
     const struct d3d12_set_root_descriptor_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetComputeRootConstantBufferView(list, args->parameter_index, args->address);
+    ID3D12GraphicsCommandList9_SetComputeRootConstantBufferView(list, args->parameter_index, args->address);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_SetComputeRootConstantBufferView(
@@ -936,7 +945,7 @@ static void d3d12_bundle_exec_set_graphics_root_cbv(d3d12_command_list_iface *li
 {
     const struct d3d12_set_root_descriptor_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetGraphicsRootConstantBufferView(list, args->parameter_index, args->address);
+    ID3D12GraphicsCommandList9_SetGraphicsRootConstantBufferView(list, args->parameter_index, args->address);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_SetGraphicsRootConstantBufferView(
@@ -957,7 +966,7 @@ static void d3d12_bundle_exec_set_compute_root_srv(d3d12_command_list_iface *lis
 {
     const struct d3d12_set_root_descriptor_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetComputeRootShaderResourceView(list, args->parameter_index, args->address);
+    ID3D12GraphicsCommandList9_SetComputeRootShaderResourceView(list, args->parameter_index, args->address);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_SetComputeRootShaderResourceView(
@@ -978,7 +987,7 @@ static void d3d12_bundle_exec_set_graphics_root_srv(d3d12_command_list_iface *li
 {
     const struct d3d12_set_root_descriptor_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetGraphicsRootShaderResourceView(list, args->parameter_index, args->address);
+    ID3D12GraphicsCommandList9_SetGraphicsRootShaderResourceView(list, args->parameter_index, args->address);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_SetGraphicsRootShaderResourceView(
@@ -999,7 +1008,7 @@ static void d3d12_bundle_exec_set_compute_root_uav(d3d12_command_list_iface *lis
 {
     const struct d3d12_set_root_descriptor_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetComputeRootUnorderedAccessView(list, args->parameter_index, args->address);
+    ID3D12GraphicsCommandList9_SetComputeRootUnorderedAccessView(list, args->parameter_index, args->address);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_SetComputeRootUnorderedAccessView(
@@ -1020,7 +1029,7 @@ static void d3d12_bundle_exec_set_graphics_root_uav(d3d12_command_list_iface *li
 {
     const struct d3d12_set_root_descriptor_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetGraphicsRootUnorderedAccessView(list, args->parameter_index, args->address);
+    ID3D12GraphicsCommandList9_SetGraphicsRootUnorderedAccessView(list, args->parameter_index, args->address);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_SetGraphicsRootUnorderedAccessView(
@@ -1045,14 +1054,14 @@ struct d3d12_ia_set_index_buffer_command
 
 static void d3d12_bundle_exec_ia_set_index_buffer_null(d3d12_command_list_iface *list, const void *args_v)
 {
-    ID3D12GraphicsCommandList6_IASetIndexBuffer(list, NULL);
+    ID3D12GraphicsCommandList9_IASetIndexBuffer(list, NULL);
 }
 
 static void d3d12_bundle_exec_ia_set_index_buffer(d3d12_command_list_iface *list, const void *args_v)
 {
     const struct d3d12_ia_set_index_buffer_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_IASetIndexBuffer(list, &args->view);
+    ID3D12GraphicsCommandList9_IASetIndexBuffer(list, &args->view);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_IASetIndexBuffer(d3d12_command_list_iface *iface,
@@ -1087,7 +1096,7 @@ static void d3d12_bundle_exec_ia_set_vertex_buffers(d3d12_command_list_iface *li
 {
     const struct d3d12_ia_set_vertex_buffers_command *args = args_v;
 
-    ID3D12GraphicsCommandList5_IASetVertexBuffers(list, args->start_slot, args->view_count, args->views);
+    ID3D12GraphicsCommandList9_IASetVertexBuffers(list, args->start_slot, args->view_count, args->views);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_IASetVertexBuffers(d3d12_command_list_iface *iface,
@@ -1202,7 +1211,7 @@ static void d3d12_bundle_exec_set_marker(d3d12_command_list_iface *list, const v
 {
     const struct d3d12_debug_marker_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetMarker(list, args->metadata, args->data, args->data_size);
+    ID3D12GraphicsCommandList9_SetMarker(list, args->metadata, args->data, args->data_size);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_SetMarker(d3d12_command_list_iface *iface,
@@ -1223,7 +1232,7 @@ static void d3d12_bundle_exec_begin_event(d3d12_command_list_iface *list, const
 {
     const struct d3d12_debug_marker_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_BeginEvent(list, args->metadata, args->data, args->data_size);
+    ID3D12GraphicsCommandList9_BeginEvent(list, args->metadata, args->data, args->data_size);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_BeginEvent(d3d12_command_list_iface *iface,
@@ -1242,7 +1251,7 @@ static void STDMETHODCALLTYPE d3d12_bundle_BeginEvent(d3d12_command_list_iface *
 
 static void d3d12_bundle_exec_end_event(d3d12_command_list_iface *list, const void *args_v)
 {
-    ID3D12GraphicsCommandList6_EndEvent(list);
+    ID3D12GraphicsCommandList9_EndEvent(list);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_EndEvent(d3d12_command_list_iface *iface)
@@ -1269,7 +1278,7 @@ static void d3d12_bundle_exec_execute_indirect(d3d12_command_list_iface *list, c
 {
     const struct d3d12_execute_indirect_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_ExecuteIndirect(list, args->signature, args->max_count,
+    ID3D12GraphicsCommandList9_ExecuteIndirect(list, args->signature, args->max_count,
             args->arg_buffer, args->arg_offset, args->count_buffer, args->count_offset);
 }
 
@@ -1331,7 +1340,7 @@ static void d3d12_bundle_exec_om_set_depth_bounds(d3d12_command_list_iface *list
 {
     const struct d3d12_om_set_depth_bounds_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_OMSetDepthBounds(list, args->min, args->max);
+    ID3D12GraphicsCommandList9_OMSetDepthBounds(list, args->min, args->max);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_OMSetDepthBounds(d3d12_command_list_iface *iface,
@@ -1360,7 +1369,7 @@ static void d3d12_bundle_exec_set_sample_positions(d3d12_command_list_iface *lis
     const struct d3d12_set_sample_positions_command *args = args_v;
 
     /* The sample position array is non-const but does not get written to */
-    ID3D12GraphicsCommandList6_SetSamplePositions(list, args->sample_count,
+    ID3D12GraphicsCommandList9_SetSamplePositions(list, args->sample_count,
             args->pixel_count, (D3D12_SAMPLE_POSITION*)args->positions);
 }
 
@@ -1403,7 +1412,7 @@ static void d3d12_bundle_exec_set_view_instance_mask(d3d12_command_list_iface *l
 {
     const struct d3d12_set_view_instance_mask_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetViewInstanceMask(list, args->mask);
+    ID3D12GraphicsCommandList9_SetViewInstanceMask(list, args->mask);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_SetViewInstanceMask(d3d12_command_list_iface *iface, UINT mask)
@@ -1429,7 +1438,7 @@ static void d3d12_bundle_exec_write_buffer_immediate(d3d12_command_list_iface *l
 {
     const struct d3d12_write_buffer_immediate_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_WriteBufferImmediate(list, args->count, args->parameters, args->modes);
+    ID3D12GraphicsCommandList9_WriteBufferImmediate(list, args->count, args->parameters, args->modes);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_WriteBufferImmediate(d3d12_command_list_iface *iface,
@@ -1525,7 +1534,7 @@ static void d3d12_bundle_exec_set_pipeline_state1(d3d12_command_list_iface *list
 {
     const struct d3d12_set_pipeline_state1_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_SetPipelineState1(list, args->state_object);
+    ID3D12GraphicsCommandList9_SetPipelineState1(list, args->state_object);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_SetPipelineState1(d3d12_command_list_iface *iface,
@@ -1550,7 +1559,7 @@ static void d3d12_bundle_exec_dispatch_rays(d3d12_command_list_iface *list, cons
 {
     const struct d3d12_dispatch_rays_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_DispatchRays(list, &args->desc);
+    ID3D12GraphicsCommandList9_DispatchRays(list, &args->desc);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_DispatchRays(d3d12_command_list_iface *iface,
@@ -1576,14 +1585,14 @@ static void d3d12_bundle_exec_rs_set_shading_rate(d3d12_command_list_iface *list
 {
     const struct d3d12_rs_set_shading_rate_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_RSSetShadingRate(list, args->base, args->combiners);
+    ID3D12GraphicsCommandList9_RSSetShadingRate(list, args->base, args->combiners);
 }
 
 static void d3d12_bundle_exec_rs_set_shading_rate_base(d3d12_command_list_iface *list, const void *args_v)
 {
     const struct d3d12_rs_set_shading_rate_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_RSSetShadingRate(list, args->base, NULL);
+    ID3D12GraphicsCommandList9_RSSetShadingRate(list, args->base, NULL);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_RSSetShadingRate(d3d12_command_list_iface *iface,
@@ -1613,7 +1622,7 @@ static void d3d12_bundle_exec_rs_set_shading_rate_image(d3d12_command_list_iface
 {
     const struct d3d12_rs_set_shading_rate_image_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_RSSetShadingRateImage(list, args->image);
+    ID3D12GraphicsCommandList9_RSSetShadingRateImage(list, args->image);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_RSSetShadingRateImage(d3d12_command_list_iface *iface,
@@ -1632,7 +1641,7 @@ static void d3d12_bundle_exec_dispatch_mesh(d3d12_command_list_iface *list, cons
 {
     const struct d3d12_dispatch_command *args = args_v;
 
-    ID3D12GraphicsCommandList6_DispatchMesh(list, args->x, args->y, args->z);
+    ID3D12GraphicsCommandList9_DispatchMesh(list, args->x, args->y, args->z);
 }
 
 static void STDMETHODCALLTYPE d3d12_bundle_DispatchMesh(d3d12_command_list_iface *iface, UINT x, UINT y, UINT z)
@@ -1648,7 +1657,92 @@ static void STDMETHODCALLTYPE d3d12_bundle_DispatchMesh(d3d12_command_list_iface
     args->z = z;
 }
 
-static CONST_VTBL struct ID3D12GraphicsCommandList6Vtbl d3d12_bundle_vtbl =
+static void STDMETHODCALLTYPE d3d12_bundle_Barrier(d3d12_command_list_iface *iface, UINT32 NumBarrierGroups, const D3D12_BARRIER_GROUP *pBarrierGroups)
+{
+    WARN("iface %p, NumBarrierGroups %u, D3D12_BARRIER_GROUP %p ignored!\n", iface, NumBarrierGroups, pBarrierGroups);
+}
+
+struct d3d12_om_set_front_and_back_stencil_ref_command
+{
+    struct d3d12_bundle_command command;
+    UINT stencil_ref_front;
+    UINT stencil_ref_back;
+};
+
+static void d3d12_bundle_exec_om_set_front_and_back_stencil_ref(d3d12_command_list_iface *list, const void *args_v)
+{
+    const struct d3d12_om_set_front_and_back_stencil_ref_command *args = args_v;
+
+    ID3D12GraphicsCommandList9_OMSetFrontAndBackStencilRef(list,
+            args->stencil_ref_front, args->stencil_ref_back);
+}
+
+static void STDMETHODCALLTYPE d3d12_bundle_OMSetFrontAndBackStencilRef(d3d12_command_list_iface *iface, UINT FrontStencilRef, UINT BackStencilRef)
+{
+    struct d3d12_bundle *bundle = impl_from_ID3D12GraphicsCommandList(iface);
+    struct d3d12_om_set_front_and_back_stencil_ref_command *args;
+
+    TRACE("iface %p, FrontStencilRef %u, BackStencilRef %u.\n", iface, FrontStencilRef, BackStencilRef);
+
+    args = d3d12_bundle_add_command(bundle, &d3d12_bundle_exec_om_set_front_and_back_stencil_ref, sizeof(*args));
+    args->stencil_ref_front = FrontStencilRef;
+    args->stencil_ref_back = BackStencilRef;
+}
+
+struct d3d12_rs_set_depth_bias_command
+{
+    struct d3d12_bundle_command command;
+    float constant_factor;
+    float clamp;
+    float slope_factor;
+};
+
+static void d3d12_bundle_exec_rs_set_depth_bias(d3d12_command_list_iface *list, const void *args_v)
+{
+    const struct d3d12_rs_set_depth_bias_command *args = args_v;
+
+    ID3D12GraphicsCommandList9_RSSetDepthBias(list,
+            args->constant_factor, args->clamp, args->slope_factor);
+}
+
+static void STDMETHODCALLTYPE d3d12_bundle_RSSetDepthBias(d3d12_command_list_iface *iface, FLOAT DepthBias, FLOAT DepthBiasClamp, FLOAT SlopeScaledDepthBias)
+{
+    struct d3d12_bundle *bundle = impl_from_ID3D12GraphicsCommandList(iface);
+    struct d3d12_rs_set_depth_bias_command *args;
+
+    TRACE("iface %p, DepthBias %f, DepthBiasClamp %f, SlopeScaledDepthBias %f.\n", iface, DepthBias, DepthBiasClamp, SlopeScaledDepthBias);
+
+    args = d3d12_bundle_add_command(bundle, &d3d12_bundle_exec_rs_set_depth_bias, sizeof(*args));
+    args->constant_factor = DepthBias;
+    args->clamp = DepthBiasClamp;
+    args->slope_factor = SlopeScaledDepthBias;
+}
+
+struct d3d12_ia_set_index_buffer_strip_cut_value_command
+{
+    struct d3d12_bundle_command command;
+    D3D12_INDEX_BUFFER_STRIP_CUT_VALUE strip_cut_value;
+};
+
+static void d3d12_bundle_exec_ia_set_index_buffer_strip_cut_value(d3d12_command_list_iface *list, const void *args_v)
+{
+    const struct d3d12_ia_set_index_buffer_strip_cut_value_command *args = args_v;
+
+    ID3D12GraphicsCommandList9_IASetIndexBufferStripCutValue(list, args->strip_cut_value);
+}
+
+static void STDMETHODCALLTYPE d3d12_bundle_IASetIndexBufferStripCutValue(d3d12_command_list_iface *iface, D3D12_INDEX_BUFFER_STRIP_CUT_VALUE IBStripCutValue)
+{
+    struct d3d12_bundle *bundle = impl_from_ID3D12GraphicsCommandList(iface);
+    struct d3d12_ia_set_index_buffer_strip_cut_value_command *args;
+
+    TRACE("iface %p, IBStripCutValue %u.\n", iface, IBStripCutValue);
+
+    args = d3d12_bundle_add_command(bundle, &d3d12_bundle_exec_ia_set_index_buffer_strip_cut_value, sizeof(*args));
+    args->strip_cut_value = IBStripCutValue;
+}
+
+static CONST_VTBL struct ID3D12GraphicsCommandList9Vtbl d3d12_bundle_vtbl =
 {
     /* IUnknown methods */
     d3d12_bundle_QueryInterface,
@@ -1741,6 +1835,13 @@ static CONST_VTBL struct ID3D12GraphicsCommandList6Vtbl d3d12_bundle_vtbl =
     d3d12_bundle_RSSetShadingRateImage,
     /* ID3D12GraphicsCommandList6 methods */
     d3d12_bundle_DispatchMesh,
+    /* ID3D12GraphicsCommandList7 methods */
+    d3d12_bundle_Barrier,
+    /* ID3D12GraphicsCommandList8 methods */
+    d3d12_bundle_OMSetFrontAndBackStencilRef,
+    /* ID3D12GraphicsCommandList9 methods */
+    d3d12_bundle_RSSetDepthBias,
+    d3d12_bundle_IASetIndexBufferStripCutValue,
 };
 
 HRESULT d3d12_bundle_create(struct d3d12_device *device,
