@@ -76,10 +76,10 @@ static const char *debug_descriptor_type(vkd3d_descriptor_qa_flags type_flags)
 
 static void vkd3d_descriptor_debug_init_once(void)
 {
-    const char *env;
+    char env[VKD3D_PATH_MAX];
+    vkd3d_get_env_var("VKD3D_DESCRIPTOR_QA_LOG", env, sizeof(env));
 
-    env = getenv("VKD3D_DESCRIPTOR_QA_LOG");
-    if (env)
+    if (strlen(env) > 0)
     {
         INFO("Enabling VKD3D_DESCRIPTOR_QA_LOG\n");
         descriptor_debug_file = fopen(env, "w");
@@ -230,7 +230,7 @@ HRESULT vkd3d_descriptor_debug_alloc_global_info(
         return hr;
     }
 
-    if (FAILED(hr = vkd3d_allocate_buffer_memory(device, global_info->vk_buffer,
+    if (FAILED(hr = vkd3d_allocate_internal_buffer_memory(device, global_info->vk_buffer,
             VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT,
             &global_info->device_allocation)))
     {
@@ -342,13 +342,18 @@ void vkd3d_descriptor_debug_register_heap(
         struct vkd3d_descriptor_qa_heap_buffer_data *heap, uint64_t cookie,
         const D3D12_DESCRIPTOR_HEAP_DESC *desc)
 {
+    unsigned int i;
     DECL_BUFFER();
 
     if (heap)
     {
         heap->num_descriptors = desc->NumDescriptors;
         heap->heap_index = cookie <= UINT32_MAX ? (uint32_t)cookie : 0u;
-        memset(heap->desc, 0, desc->NumDescriptors * sizeof(*heap->desc));
+        for (i = 0; i < desc->NumDescriptors; i++)
+        {
+            heap->desc[i].cookie = 0;
+            heap->desc[i].descriptor_type = ~0u;
+        }
     }
 
     if (!vkd3d_descriptor_debug_active_log())
@@ -394,6 +399,52 @@ void vkd3d_descriptor_debug_unregister_heap(uint64_t cookie)
     FLUSH_BUFFER();
 }
 
+void vkd3d_descriptor_debug_register_query_heap_cookie(struct vkd3d_descriptor_qa_global_info *global_info,
+        uint64_t cookie, const D3D12_QUERY_HEAP_DESC *desc)
+{
+    const char *type_desc;
+    DECL_BUFFER();
+
+    if (!vkd3d_descriptor_debug_active_log())
+        return;
+
+    APPEND_SNPRINTF("QUERY HEAP CREATE #%"PRIu64" || ", cookie);
+
+    switch (desc->Type)
+    {
+        case D3D12_QUERY_HEAP_TYPE_COPY_QUEUE_TIMESTAMP:
+            type_desc = "COPY_QUEUE_TIMESTAMP";
+            break;
+
+        case D3D12_QUERY_HEAP_TYPE_OCCLUSION:
+            type_desc = "OCCLUSION";
+            break;
+
+        case D3D12_QUERY_HEAP_TYPE_TIMESTAMP:
+            type_desc = "TIMESTAMP";
+            break;
+
+        case D3D12_QUERY_HEAP_TYPE_VIDEO_DECODE_STATISTICS:
+            type_desc = "VIDEO_DECODE_STATISTICS";
+            break;
+
+        case D3D12_QUERY_HEAP_TYPE_PIPELINE_STATISTICS:
+            type_desc = "PIPELINE_STATISTICS";
+            break;
+
+        case D3D12_QUERY_HEAP_TYPE_SO_STATISTICS:
+            type_desc = "SO_STATISTICS";
+            break;
+
+        default:
+            type_desc = "???";
+            break;
+    }
+
+    APPEND_SNPRINTF(" || %s || Count = %u", type_desc, desc->Count);
+    FLUSH_BUFFER();
+}
+
 void vkd3d_descriptor_debug_register_resource_cookie(struct vkd3d_descriptor_qa_global_info *global_info,
         uint64_t cookie, const D3D12_RESOURCE_DESC1 *desc)
 {
