@@ -33,7 +33,9 @@
 #include "vkd3d_windows.h"
 #define WIDL_C_INLINE_WRAPPERS
 #include "vkd3d_d3d12.h"
+#include "vkd3d_device_vkd3d_ext.h"
 #include "vkd3d_d3d12sdklayers.h"
+#undef WIDL_C_INLINE_WRAPPERS
 
 #include <inttypes.h>
 #include <limits.h>
@@ -43,13 +45,14 @@
 #if !defined(_WIN32) || defined(VKD3D_FORCE_UTILS_WRAPPER)
 # include "vkd3d_threads.h"
 # include "vkd3d.h"
-# include "vkd3d_utils.h"
 # include "vkd3d_sonames.h"
 #endif
 
 #if !defined(_WIN32)
 #include <dlfcn.h>
 #include <unistd.h>
+#include <sys/poll.h>
+#include <sys/eventfd.h>
 #endif
 
 #include "d3d12_test_utils.h"
@@ -59,28 +62,10 @@ extern PFN_D3D12_ENABLE_EXPERIMENTAL_FEATURES pfn_D3D12EnableExperimentalFeature
 extern PFN_D3D12_GET_DEBUG_INTERFACE pfn_D3D12GetDebugInterface;
 extern PFN_D3D12_CREATE_VERSIONED_ROOT_SIGNATURE_DESERIALIZER pfn_D3D12CreateVersionedRootSignatureDeserializer;
 extern PFN_D3D12_SERIALIZE_VERSIONED_ROOT_SIGNATURE pfn_D3D12SerializeVersionedRootSignature;
+extern bool use_warp_device;
+extern unsigned int use_adapter_idx;
 
 #if defined(_WIN32) && !defined(VKD3D_FORCE_UTILS_WRAPPER)
-static inline HANDLE create_event(void)
-{
-    return CreateEventA(NULL, FALSE, FALSE, NULL);
-}
-
-static inline void signal_event(HANDLE event)
-{
-    SetEvent(event);
-}
-
-static inline unsigned int wait_event(HANDLE event, unsigned int milliseconds)
-{
-    return WaitForSingleObject(event, milliseconds);
-}
-
-static inline void destroy_event(HANDLE event)
-{
-    CloseHandle(event);
-}
-
 #define get_d3d12_pfn(name) get_d3d12_pfn_(#name)
 static inline void *get_d3d12_pfn_(const char *name)
 {
@@ -90,31 +75,95 @@ static inline void *get_d3d12_pfn_(const char *name)
     return GetProcAddress(d3d12_module, name);
 }
 #else
-#define INFINITE VKD3D_INFINITE
-#define WAIT_OBJECT_0 VKD3D_WAIT_OBJECT_0
-#define WAIT_TIMEOUT VKD3D_WAIT_TIMEOUT
+#define get_d3d12_pfn(name) (name)
+#endif
 
+#if defined(_WIN32)
 static inline HANDLE create_event(void)
 {
-    return vkd3d_create_event();
-}
-
-static inline void signal_event(HANDLE event)
-{
-    vkd3d_signal_event(event);
+    return CreateEventA(NULL, FALSE, FALSE, NULL);
 }
 
 static inline unsigned int wait_event(HANDLE event, unsigned int milliseconds)
 {
-    return vkd3d_wait_event(event, milliseconds);
+    return WaitForSingleObject(event, milliseconds);
+}
+
+static inline void signal_event(HANDLE event)
+{
+    SetEvent(event);
 }
 
 static inline void destroy_event(HANDLE event)
 {
-    vkd3d_destroy_event(event);
+    CloseHandle(event);
+}
+#else
+#define INFINITE INT_MAX
+#define WAIT_OBJECT_0 0
+#define WAIT_TIMEOUT 1
+#define WAIT_TIMEOUT 1
+
+static inline HANDLE create_event(void)
+{
+    HANDLE handle;
+    int fd;
+
+    fd = eventfd(0, EFD_CLOEXEC | EFD_NONBLOCK);
+    if (fd < 0)
+        return NULL;
+
+    /* No way this should happen unless stdin is closed for some reason ...
+     * When casting to a HANDLE null will be considered no handle. */
+    if (fd == 0)
+    {
+        fd = dup(0);
+        close(0);
+    }
+
+    handle = (HANDLE)(intptr_t)fd;
+    return handle;
 }
 
-#define get_d3d12_pfn(name) (name)
+static inline unsigned int wait_event(HANDLE event, unsigned int milliseconds)
+{
+    int fd = (int)(intptr_t)event;
+    struct pollfd pfd;
+    uint64_t dummy;
+    int timeout;
+
+    TRACE("event %p, milliseconds %u.\n", event, milliseconds);
+
+    pfd.events = POLLIN;
+    pfd.fd = fd;
+
+    timeout = milliseconds == INFINITE ? -1 : (int)milliseconds;
+
+    for (;;)
+    {
+        if (poll(&pfd, 1, timeout) <= 0)
+            return WAIT_TIMEOUT;
+
+        /* Non-blocking reads, if there are two racing threads that wait on a Win32 event,
+         * only one will succeed when auto-reset events are used. */
+        if (read(fd, &dummy, sizeof(dummy)) > 0)
+            return WAIT_OBJECT_0;
+        else if (timeout != INFINITE)
+            return WAIT_TIMEOUT;
+    }
+}
+
+static inline void signal_event(HANDLE event)
+{
+    int fd = (int)(intptr_t)event;
+    const uint64_t value = 1;
+    write(fd, &value, sizeof(value));
+}
+
+static inline void destroy_event(HANDLE event)
+{
+    close((int)(intptr_t)event);
+}
 #endif
 
 #if defined(_WIN32)
@@ -213,6 +262,48 @@ static inline bool join_thread(HANDLE untyped_thread)
 }
 #endif
 
+extern D3D_FEATURE_LEVEL vkd3d_device_feature_level;
+static inline void enable_feature_level_override(int argc, char **argv)
+{
+    static const struct
+    {
+        D3D_FEATURE_LEVEL level;
+        const char *tag;
+    } level_map[] = {
+        { D3D_FEATURE_LEVEL_11_0, "11_0" },
+        { D3D_FEATURE_LEVEL_11_1, "11_1" },
+        { D3D_FEATURE_LEVEL_12_0, "12_0" },
+        { D3D_FEATURE_LEVEL_12_1, "12_1" },
+        { D3D_FEATURE_LEVEL_12_2, "12_2" },
+    };
+
+    const char *level = NULL;
+    int i;
+
+    for (i = 1; i + 1 < argc; ++i)
+    {
+        if (!strcmp(argv[i], "--feature-level"))
+        {
+            level = argv[i + 1];
+            break;
+        }
+    }
+
+    vkd3d_device_feature_level = D3D_FEATURE_LEVEL_11_0;
+    if (level)
+    {
+        for (i = 0; i < (int)ARRAY_SIZE(level_map); i++)
+        {
+            if (!strcmp(level_map[i].tag, level))
+            {
+                INFO("Overriding feature level %s.\n", level);
+                vkd3d_device_feature_level = level_map[i].level;
+                break;
+            }
+        }
+    }
+}
+
 static HRESULT wait_for_fence(ID3D12Fence *fence, uint64_t value)
 {
     unsigned int ret;
@@ -279,9 +370,6 @@ static inline void wait_queue_idle_no_event_(unsigned int line, ID3D12Device *de
     ID3D12Fence_Release(fence);
 }
 
-static bool use_warp_device;
-static unsigned int use_adapter_idx;
-
 #if defined(_WIN32) && !defined(VKD3D_FORCE_UTILS_WRAPPER)
 static IUnknown *create_warp_adapter(IDXGIFactory4 *factory)
 {
@@ -333,7 +421,7 @@ static ID3D12Device *create_device(void)
     if (pfn_D3D12EnableExperimentalFeatures)
         pfn_D3D12EnableExperimentalFeatures(1, &D3D12ExperimentalShaderModels, NULL, NULL);
 
-    hr = pfn_D3D12CreateDevice(adapter, D3D_FEATURE_LEVEL_11_0, &IID_ID3D12Device, (void **)&device);
+    hr = pfn_D3D12CreateDevice(adapter, vkd3d_device_feature_level, &IID_ID3D12Device, (void **)&device);
     if (adapter)
         IUnknown_Release(adapter);
 
@@ -418,6 +506,13 @@ static inline bool is_intel_windows_device(ID3D12Device *device)
     return get_adapter_desc(device, &desc) && desc.VendorId == 0x8086;
 }
 
+static inline bool is_nvidia_windows_device(ID3D12Device *device)
+{
+    DXGI_ADAPTER_DESC desc;
+
+    return get_adapter_desc(device, &desc) && desc.VendorId == 0x10de;
+}
+
 static inline bool is_mesa_device(ID3D12Device *device)
 {
     return false;
@@ -438,11 +533,10 @@ static inline bool is_radv_device(ID3D12Device *device)
     return false;
 }
 
-static inline bool is_depth_clip_enable_supported(ID3D12Device *device)
+static inline bool is_amd_vulkan_device(ID3D12Device *device)
 {
-    return true;
+    return false;
 }
-
 #else
 
 static PFN_vkGetInstanceProcAddr pfn_vkGetInstanceProcAddr;
@@ -474,131 +568,11 @@ static bool init_vulkan_loader(void)
     return pfn_vkGetInstanceProcAddr != NULL;
 }
 
-#define get_vk_instance_proc(instance, sym) (PFN_##sym)get_vk_instance_proc_(instance, #sym)
-static PFN_vkVoidFunction get_vk_instance_proc_(VkInstance instance, const char *sym)
-{
-    if (!init_vulkan_loader())
-        return NULL;
-
-    return pfn_vkGetInstanceProcAddr(instance, sym);
-}
-
-static bool check_device_extension(VkInstance instance, VkPhysicalDevice vk_physical_device, const char *name)
-{
-    VkExtensionProperties *properties;
-    bool ret = false;
-    unsigned int i;
-    uint32_t count;
-    VkResult vr;
-
-    PFN_vkEnumerateDeviceExtensionProperties pfn_vkEnumerateDeviceExtensionProperties;
-    pfn_vkEnumerateDeviceExtensionProperties = get_vk_instance_proc(instance, vkEnumerateDeviceExtensionProperties);
-    if (!pfn_vkEnumerateDeviceExtensionProperties)
-        return false;
-
-    vr = pfn_vkEnumerateDeviceExtensionProperties(vk_physical_device, NULL, &count, NULL);
-    ok(vr == VK_SUCCESS, "Got unexpected VkResult %d.\n", vr);
-    if (!count)
-        return false;
-
-    properties = calloc(count, sizeof(*properties));
-    ok(properties, "Failed to allocate memory.\n");
-
-    vr = pfn_vkEnumerateDeviceExtensionProperties(vk_physical_device, NULL, &count, properties);
-    ok(vr == VK_SUCCESS, "Got unexpected VkResult %d.\n", vr);
-    for (i = 0; i < count; ++i)
-    {
-        if (!strcmp(properties[i].extensionName, name))
-        {
-            ret = true;
-            break;
-        }
-    }
-
-    free(properties);
-    return ret;
-}
-
-static HRESULT create_vkd3d_instance(struct vkd3d_instance **instance)
-{
-    struct vkd3d_instance_create_info instance_create_info = {
-        .pfn_signal_event = vkd3d_signal_event,
-    };
-
-    return vkd3d_create_instance(&instance_create_info, instance);
-}
-
-static VkPhysicalDevice select_physical_device(struct vkd3d_instance *instance)
-{
-    VkPhysicalDevice *vk_physical_devices;
-    VkPhysicalDevice vk_physical_device;
-    VkInstance vk_instance;
-    uint32_t count;
-    VkResult vr;
-    PFN_vkEnumeratePhysicalDevices pfn_vkEnumeratePhysicalDevices;
-    vk_instance = vkd3d_instance_get_vk_instance(instance);
-
-    pfn_vkEnumeratePhysicalDevices = get_vk_instance_proc(vk_instance, vkEnumeratePhysicalDevices);
-    if (!pfn_vkEnumeratePhysicalDevices)
-        return VK_NULL_HANDLE;
-
-    vr = pfn_vkEnumeratePhysicalDevices(vk_instance, &count, NULL);
-    ok(vr == VK_SUCCESS, "Got unexpected VkResult %d.\n", vr);
-
-    if (use_adapter_idx >= count)
-    {
-        trace("Invalid physical device index %u.\n", use_adapter_idx);
-        return VK_NULL_HANDLE;
-    }
-
-    vk_physical_devices = calloc(count, sizeof(*vk_physical_devices));
-    ok(vk_physical_devices, "Failed to allocate memory.\n");
-    vr = pfn_vkEnumeratePhysicalDevices(vk_instance, &count, vk_physical_devices);
-    ok(vr == VK_SUCCESS, "Got unexpected VkResult %d.\n", vr);
-
-    vk_physical_device = vk_physical_devices[use_adapter_idx];
-
-    free(vk_physical_devices);
-
-    return vk_physical_device;
-}
-
-static HRESULT create_vkd3d_device(struct vkd3d_instance *instance,
-        D3D_FEATURE_LEVEL minimum_feature_level, REFIID iid, void **device)
-{
-    static const char * const device_extensions[] =
-    {
-        VK_KHR_DRIVER_PROPERTIES_EXTENSION_NAME,
-    };
-
-    struct vkd3d_device_create_info device_create_info;
-    VkPhysicalDevice vk_physical_device;
-
-    if (!(vk_physical_device = select_physical_device(instance)))
-        return E_INVALIDARG;
-
-    memset(&device_create_info, 0, sizeof(device_create_info));
-    device_create_info.minimum_feature_level = minimum_feature_level;
-    device_create_info.instance = instance;
-    device_create_info.vk_physical_device = vk_physical_device;
-    device_create_info.optional_device_extensions = device_extensions;
-    device_create_info.optional_device_extension_count = ARRAY_SIZE(device_extensions);
-
-    return vkd3d_create_device(&device_create_info, iid, device);
-}
-
 static ID3D12Device *create_device(void)
 {
-    struct vkd3d_instance *instance;
     ID3D12Device *device;
     HRESULT hr;
-
-    if (SUCCEEDED(hr = create_vkd3d_instance(&instance)))
-    {
-        hr = create_vkd3d_device(instance, D3D_FEATURE_LEVEL_11_0, &IID_ID3D12Device, (void **)&device);
-        vkd3d_instance_decref(instance);
-    }
-
+    hr = D3D12CreateDevice(NULL, vkd3d_device_feature_level, &IID_ID3D12Device, (void **)&device);
     return SUCCEEDED(hr) ? device : NULL;
 }
 
@@ -608,54 +582,43 @@ static bool get_driver_properties(ID3D12Device *device, VkPhysicalDeviceDriverPr
     VkPhysicalDeviceProperties2 device_properties2;
     VkPhysicalDevice vk_physical_device;
     VkInstance vk_instance;
-
-    memset(driver_properties, 0, sizeof(*driver_properties));
-    driver_properties->sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DRIVER_PROPERTIES_KHR;
-
-    vk_physical_device = vkd3d_get_vk_physical_device(device);
-    vk_instance = vkd3d_instance_get_vk_instance(vkd3d_instance_from_device(device));
+    ID3D12DeviceExt *ext;
+    VkDevice vk_device;
 
     if (!init_vulkan_loader())
         return false;
 
-    if (check_device_extension(vk_instance, vk_physical_device, VK_KHR_DRIVER_PROPERTIES_EXTENSION_NAME))
-    {
-        struct vkd3d_instance *instance = vkd3d_instance_from_device(device);
-        VkInstance vk_instance = vkd3d_instance_get_vk_instance(instance);
+    if (FAILED(ID3D12Device_QueryInterface(device, &IID_ID3D12DeviceExt, (void **)&ext)))
+        return false;
 
-        pfn_vkGetPhysicalDeviceProperties2
-                = (void *)pfn_vkGetInstanceProcAddr(vk_instance, "vkGetPhysicalDeviceProperties2");
-        ok(pfn_vkGetPhysicalDeviceProperties2, "vkGetPhysicalDeviceProperties2 is NULL.\n");
+    ID3D12DeviceExt_GetVulkanHandles(ext, &vk_instance, &vk_physical_device, &vk_device);
+	ID3D12DeviceExt_Release(ext);
 
-        memset(&device_properties2, 0, sizeof(device_properties2));
-        device_properties2.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROPERTIES_2;
-        device_properties2.pNext = driver_properties;
-        pfn_vkGetPhysicalDeviceProperties2(vk_physical_device, &device_properties2);
-        return true;
-    }
+    memset(driver_properties, 0, sizeof(*driver_properties));
+    driver_properties->sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DRIVER_PROPERTIES_KHR;
 
-    return false;
+    pfn_vkGetPhysicalDeviceProperties2
+        = (void *)pfn_vkGetInstanceProcAddr(vk_instance, "vkGetPhysicalDeviceProperties2");
+    ok(pfn_vkGetPhysicalDeviceProperties2, "vkGetPhysicalDeviceProperties2 is NULL.\n");
+
+    memset(&device_properties2, 0, sizeof(device_properties2));
+    device_properties2.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROPERTIES_2;
+    device_properties2.pNext = driver_properties;
+    pfn_vkGetPhysicalDeviceProperties2(vk_physical_device, &device_properties2);
+    return true;
 }
 
 static inline void init_adapter_info(void)
 {
     VkPhysicalDeviceDriverPropertiesKHR driver_properties;
-    struct vkd3d_instance *instance;
     ID3D12Device *device;
-    HRESULT hr;
 
-    if (FAILED(hr = create_vkd3d_instance(&instance)))
-        return;
-
-    if (SUCCEEDED(hr = create_vkd3d_device(instance, D3D_FEATURE_LEVEL_11_0, &IID_ID3D12Device, (void **)&device)))
+    if ((device = create_device()))
     {
         if (get_driver_properties(device, &driver_properties))
             trace("Driver name: %s, driver info: %s.\n", driver_properties.driverName, driver_properties.driverInfo);
-
         ID3D12Device_Release(device);
     }
-
-    vkd3d_instance_decref(instance);
 }
 
 static inline bool is_amd_windows_device(ID3D12Device *device)
@@ -668,6 +631,11 @@ static inline bool is_intel_windows_device(ID3D12Device *device)
     return false;
 }
 
+static inline bool is_nvidia_windows_device(ID3D12Device *device)
+{
+    return false;
+}
+
 static inline bool is_mesa_device(ID3D12Device *device)
 {
     VkPhysicalDeviceDriverPropertiesKHR properties;
@@ -701,12 +669,14 @@ static inline bool is_radv_device(ID3D12Device *device)
     return properties.driverID == VK_DRIVER_ID_MESA_RADV_KHR;
 }
 
-static inline bool is_depth_clip_enable_supported(ID3D12Device *device)
+static inline bool is_amd_vulkan_device(ID3D12Device *device)
 {
-    struct vkd3d_instance *instance = vkd3d_instance_from_device(device);
-    VkInstance vk_instance = vkd3d_instance_get_vk_instance(instance);
-    VkPhysicalDevice vk_physical_device = vkd3d_get_vk_physical_device(device);
-    return check_device_extension(vk_instance, vk_physical_device, VK_EXT_DEPTH_CLIP_ENABLE_EXTENSION_NAME);
+    VkPhysicalDeviceDriverPropertiesKHR properties;
+
+    get_driver_properties(device, &properties);
+    return properties.driverID == VK_DRIVER_ID_MESA_RADV_KHR ||
+            properties.driverID == VK_DRIVER_ID_AMD_OPEN_SOURCE_KHR ||
+            properties.driverID == VK_DRIVER_ID_AMD_PROPRIETARY;
 }
 #endif
 
